@@ -3,7 +3,7 @@
 Copyright (C) 1997 Fabrice POPINEAU.
 Adapted to MS-DOS/DJGPP by Eli Zaretskii <eliz@is.elta.co.il>.
 
-Time-stamp: <99/01/26 14:19:27 popineau>
+Time-stamp: <99/06/01 22:32:05 popineau>
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
@@ -40,8 +40,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 */
 
 /* mktex.opt */
-int program_number;
-string progname;
+int program_number = -1;
+string progname = NULL;
 
 /* mktex.cnf */
 
@@ -67,6 +67,7 @@ program_description makedesc [] = {
   {"mktextex", 4, 6, mktextex },	/* ?? */
   {"mktexmf",  2, 2, mktexmf },		/* 1,1 */
   {"mktexlsr", 1, 255, mktexlsr },	/* 0,MANY */
+  {"texhash", 1, 255, mktexlsr },	/* 0,MANY */
   {"mktexupd", 3, 3, mktexupdate },	/* 2,2 */
   {"mktexdir", 2, 255, mktexmkdir },	/* 0,MANY */
   {"mktexrmd", 2, 255, mktexrmdir },	/* ?? */
@@ -148,6 +149,15 @@ static string usage_msg[][MAX_LINES] = {
     "($TEXMFDBS) are used.\n",
     0
   },
+  /* TeXHash */
+  {
+    "Usage: %s [DIRS ...]\n\n",
+    "Rebuild all necessary ls-R filename databases completely. If one or\n",
+    "more arguments DIRS are given, these are used as texmf directories to\n",
+    "build ls-R for. Else all directories in the search path for ls-R files\n",
+    "($TEXMFDBS) are used.\n",
+    0
+  },
   /* Mktexupd */
   {
     "Usage : %s DIR FILE.\n",
@@ -183,8 +193,8 @@ static string usage_msg[][MAX_LINES] = {
 /*
   First part: mktex.opt
   */
- int mktex_opt(int argc, char *argv[],
-			program_description * program)
+int mktex_opt(int argc, char *argv[],
+	      program_description * program)
 {
   int g; /* getopt return code */
   int i;
@@ -197,14 +207,15 @@ static string usage_msg[][MAX_LINES] = {
     if (g == EOF)
       break;
 
-    if (g == '?')
-      return 1;  /* Unknown option.  */
+    if (g == '?') {
+      usage();			/* Unknown option.  */
+      exit(1);
+    }
 
     /* assert (g == 0); */ /* We have no short option names.  */
     /*
       FIXME : try 'mktexpk --mfmode --bdpi 600 ...'
       */
-    
     if (ARGUMENT_IS ("debug")) {
       kpathsea_debug |= atoi (optarg);
     }
@@ -450,7 +461,7 @@ void mktexinit(int argc, char **argv)
   /* This is unused.  */
   program_description * program = 0;
 #endif
-  
+
   program_number = -1;
 
   if (!progname)
@@ -1281,24 +1292,29 @@ void do_mktexnames(string name)
       /* Try Sauter font */
       fullname = find_sauter(name, &rootname);
       if (test_file('z', fullname)) {
-	if (FILESTRNCASEEQ(rootname, "wn", 2)
-	    || FILESTRNCASEEQ(rootname, "lh", 2)
-	    || FILESTRNCASEEQ(rootname, "ll", 2)
-	    || FILESTRNCASEEQ(rootname, "rx", 2)
-	    || FILESTRNCASEEQ(rootname, "la", 2)
-	    || FILESTRNCASEEQ(rootname, "lb", 2)
-	    || FILESTRNCASEEQ(rootname, "lc", 2)
-	    ) {
+	if (FILESTRNCASEEQ(rootname, "cs", 2)
+		 || FILESTRNCASEEQ(rootname, "lcsss", 5)
+		 || FILESTRNCASEEQ(rootname, "icscsc", 6)
+		 || FILESTRNCASEEQ(rootname, "icstt", 5)
+		 || FILESTRNCASEEQ(rootname, "ilcsss", 6)
+		 ) {
+	  fullname = kpse_find_file("cscode.mf", kpse_mf_format, false);	  
+	}
+	else if (strlen(rootname) >= 3
+		 && ((FILESTRNCASEEQ(rootname, "wn", 2)
+		      && strchr("bBcCdDfFiIrRsStTuUvV", rootname[2]))
+		     || (FILESTRNCASEEQ(rootname, "rx", 2)
+			 && strchr("bBcCdDfFiIoOrRsStTuUvVxX", rootname[2])
+			 && strlen(rootname) >= 4
+			 && strchr("bBcCfFhHiIlLmMoOsStTxX", rootname[3]))
+		     || ((rootname[0] == 'l' || rootname[0] == 'L')
+			 && strchr("aAbBcChHlL", rootname[1])
+			 && strchr("bBcCdDfFiIoOrRsStTuUvVxX", rootname[2])))) {
 	  char lhname[64];
 	  strcpy(lhname, "xxcodes.mf");
 	  strncpy(lhname, name, 2);
 	  fullname = kpse_find_file(lhname, kpse_mf_format, false);
 	}
-	else if (FILESTRNCASEEQ(rootname, "wn", 2)
-		 || FILESTRNCASEEQ(rootname, "lcs", 3)
-		 || FILESTRNCASEEQ(rootname, "ics", 3)) {
-	  fullname = kpse_find_file("cscode.mf", kpse_mf_format, false);	  
-	}
 	else {
 	  fullname = kpse_find_file(rootname, kpse_mf_format, false);
 	}
@@ -1307,7 +1323,8 @@ void do_mktexnames(string name)
   }
 
   if (test_file('z', fullname)) {
-    setval("MT_FEATURES", concat(getval("MT_FEATURES"), ":fontmaps"));
+    /* Moved on 99/04/29
+       setval("MT_FEATURES", concat(getval("MT_FEATURES"), ":fontmaps")); */
     setval_default("MT_DESTROOT", getval("MT_VARTEXFONTS"));
   }
   else {
@@ -1440,6 +1457,14 @@ void do_mktexnames(string name)
   setval_default("MT_TFMDESTREL", getval("MT_DEFAULT_TFMDESTREL"));
   setval_default("MT_MFDESTREL", getval("MT_DEFAULT_MFDESTREL"));
   
+  /* We have found nothing, so force using the fontmaps as a last resort.
+     This also means mktexnam can be queried for advice on where to
+     place fonts. */
+  if (STREQ(getval("MT_SUPPLIER"), "unknown")
+      || STREQ(getval("MT_TYPEFACE"), "unknown")) {
+    setval_default("MT_DESTROOT", getval("MT_VARTEXFONTS"));
+  }
+
   /* Handle the options */
   mktexnam_opt();
 
@@ -1718,28 +1743,33 @@ void do_mktexmf(string font)
 
   if (test_file('n', sauterroot)) {
     rootfile = sauterroot;
+    rootname = concat("b-", rootname);
   }
-  else if (FILESTRNCASEEQ(rootname, "wn", 2)
-	   || FILESTRNCASEEQ(rootname, "lh", 2)
-	   || FILESTRNCASEEQ(rootname, "ll", 2)
-	   || FILESTRNCASEEQ(rootname, "rx", 2)
-	   || FILESTRNCASEEQ(rootname, "la", 2)
-	   || FILESTRNCASEEQ(rootname, "lb", 2)
-	|| FILESTRNCASEEQ(rootname, "lc", 2)
+  else if (FILESTRNCASEEQ(rootname, "cs", 2)
+	   || FILESTRNCASEEQ(rootname, "lcsss", 5)
+	   || FILESTRNCASEEQ(rootname, "icscsc", 6)
+	   || FILESTRNCASEEQ(rootname, "icstt", 5)
+	   || FILESTRNCASEEQ(rootname, "ilcsss", 6)
 	   ) {
+    rootfile = kpse_find_file("cscode.mf",
+			      kpse_mf_format, false);
+  }
+  else if (strlen(rootname) >= 3
+	   && ((FILESTRNCASEEQ(rootname, "wn", 2)
+		&& strchr("bBcCdDfFiIrRsStTuUvV", rootname[2]))
+	       || (FILESTRNCASEEQ(rootname, "rx", 2)
+		   && strchr("bBcCdDfFiIoOrRsStTuUvVxX", rootname[2])
+		   && strlen(rootname) >= 4
+		   && strchr("bBcCfFhHiIlLmMoOsStTxX", rootname[3]))
+	       || ((rootname[0] == 'l' || rootname[0] == 'L')
+		   && strchr("aAbBcChHlL", rootname[1])
+		   && strchr("bBcCdDfFiIoOrRsStTuUvVxX", rootname[2])))) {
     char lhprefix[64];
     strcpy(lhprefix, "xxcodes.mf");
     strncpy(lhprefix, name, 2);
     rootfile = kpse_find_file(lhprefix,
 			      kpse_mf_format, false);
   }
-  else if (FILESTRNCASEEQ(rootname, "cs", 2)
-	   || FILESTRNCASEEQ(rootname, "ics", 3)
-	   || FILESTRNCASEEQ(rootname, "lcs", 3)
-	   ) {
-    rootfile = kpse_find_file("cscode.mf",
-			      kpse_mf_format, false);
-  }
   else {
     string tem;
     rootfile = kpse_find_file(tem = concat(rootname, ".mf"),
@@ -1770,7 +1800,7 @@ void do_mktexmf(string font)
     fprintf(stderr, "%s: no pointsize.\n", progname);
     mt_exit(1);
   } else if (ptsz_len == 2) {
-    if (pointsize[0] == '1' && pointsize[1] == '2')
+    if (pointsize[0] == '1' && pointsize[1] == '1')
       realsize = "10.95";		/* \magstephalf */
     else if (pointsize[0] == '1' && pointsize[1] == '4')
       realsize = "14.4";		/* \magstep2 */
@@ -1841,25 +1871,39 @@ void do_mktexmf(string font)
     fprintf(f, "gensize:=%s;\ngenerate %s;\n", realsize, rootname);
 
   }
-  else if (FILESTRNCASEEQ(name, "wn", 2)
-	|| FILESTRNCASEEQ(name, "lh", 2)
-	|| FILESTRNCASEEQ(name, "ll", 2)
-	|| FILESTRNCASEEQ(name, "rx", 2)
-	|| FILESTRNCASEEQ(name, "la", 2)
-	|| FILESTRNCASEEQ(name, "lb", 2)
-	|| FILESTRNCASEEQ(name, "lc", 2)
+  else if (FILESTRNCASEEQ(name, "cs", 2)
+	   || FILESTRNCASEEQ(name, "lcsss", 5)
+	   || FILESTRNCASEEQ(name, "icscsc", 6)
+	   || FILESTRNCASEEQ(name, "icstt", 5)
+	   || FILESTRNCASEEQ(name, "ilcsss", 6)
 	   ) {
+    fprintf(f, "input cscode\nuse_driver;\n");
+  }
+  else if (strlen(name) >= 3
+	   && ((FILESTRNCASEEQ(name, "wn", 2)
+		&& strchr("bBcCdDfFiIrRsStTuUvV", name[2]))
+	       || (FILESTRNCASEEQ(name, "rx", 2)
+		   && strchr("bBcCdDfFiIoOrRsStTuUvVxX", name[2])
+		   && strlen(name) >= 4
+		   && strchr("bBcCfFhHiIlLmMoOsStTxX", name[3]))
+	       || ((name[0] == 'l' || name[0] == 'L')
+		   && strchr("aAbBcChHlL", name[1])
+		   && strchr("bBcCdDfFiIoOrRsStTuUvVxX", name[2])))) {
     fprintf(f, "input fikparm;\n");
   }
-  else if (FILESTRNCASEEQ(name, "cs", 2)
-	   || FILESTRNCASEEQ(name, "ics", 3)
-	   || FILESTRNCASEEQ(name, "lcs", 3)) {
-    fprintf(f, "input cscode\nuse_driver;\n");
+  else if (strlen(name) >= 4 && strchr("gG", name[0])
+	   && strchr("oOrRsStTlL", name[1]) 
+	   && strchr("iIjJmMtTwWxX", name[2]) 
+	   && strchr("cCiInNoOuU", name[3])) {
+    /* The CB font family; the test matches (case insensitively) the
+       pattern `g[lorst][ijmtwx][cinou]*'. 
+       This is only slightly more general than the exact set of patterns.  */
+    fprintf(f, "gensize:=%s;\ninput %s;\n", realsize, rootname);
   }
   else {
     /* FIXME: this was in the previous versions */
     /*      fprintf(tmpfile, "if unknown %s: input %s fi;\n", base, base); */
-      fprintf(f, "design_size := %s;\ninput b-%s;\n",
+      fprintf(f, "design_size := %s;\ninput %s;\n",
 	      realsize, rootname);
   }
   fclose(f);
@@ -1879,7 +1923,7 @@ void do_mktexmf(string font)
 
 int mktexmf(int argc, char **argv)
 {
-  start_redirection();
+  start_redirection(false);
   do_mktexmf(argv[1]);
   return 0;
 }
@@ -1893,7 +1937,8 @@ boolean check_for_strange_path(string name)
 {
   FILE *f;
   string line;
-  string strange = "! Strange path";
+  string strange1 = "! Strange path";
+  string strange2 = "! bad pos";
   boolean res = false;	/* in case there are no lines which begin with `!' */
 
   if ((f = fopen(name, "r")) == NULL) {
@@ -1902,7 +1947,8 @@ boolean check_for_strange_path(string name)
   }
   while ((line  = read_line(f)) != NULL) {
     if (*line == '!') {
-      if (strncmp(line, strange, strlen(strange))) {
+      if (strncmp(line, strange1, strlen(strange1))
+	  && strncmp(line, strange2, strlen(strange2))) {
 	res = false;
 	break;
       }
@@ -1964,10 +2010,11 @@ void do_mktextfm(string font, string destdir)
 
   /* Now run metafont.
      Note that stdin has been redirected to "/dev/null" by `main'. */
-  retcode = system(cmd);
+  retcode = win32_system(cmd);
+
   if (retcode != 0) {
     if (retcode == -1) {
-      perror("mf");
+      perror(cmd);
       mt_exit(1);
     }
     else if (check_for_strange_path(expand_var("$NAME.log")) == true) {
@@ -2029,7 +2076,7 @@ void do_mktextfm(string font, string destdir)
     if (KPSE_DEBUG_P(MKTEX_DEBUG)) {
       fprintf(stderr, "Running: %s\n", cmd);
     }
-    retcode = system(cmd);
+    retcode = win32_system(cmd);
     if (retcode != 0) {
       if (retcode == -1)
 	perror("gftopk");
@@ -2066,7 +2113,18 @@ int mktextfm(int argc, char **argv)
   string dest = getval("DEST");
   string font = argv[1];
 
-  start_redirection();
+  /* Where do potential driver files belong?
+     case "$MT_FEATURES" in
+     *nomfdrivers*)
+     : ${MT_MFDESTDIR=`pwd`}
+     export MT_MFDESTDIR;;
+     esac
+  */
+  if (feature_p("nomfdrivers")) {
+    setval_default("MT_FEATURES", xgetcwd());
+  }
+
+  start_redirection(false);
   do_mktextfm(font, dest);
   return 0;
 }
@@ -2134,6 +2192,17 @@ int mktexpk(int argc, char **argv)
   int mf_bdpi, ibdpi, retcode;
   boolean has_ttf2pk = false, has_hbf2gf = false;
 
+  /* Where do potential driver files belong?
+     case "$MT_FEATURES" in
+     *nomfdrivers*)
+     : ${MT_MFDESTDIR=`pwd`}
+     export MT_MFDESTDIR;;
+     esac
+  */
+  if (feature_p("nomfdrivers")) {
+    setval_default("MT_FEATURES", xgetcwd());
+  }
+
   if (argc == 3) {
     char *ep = NULL;
     int fd = - 1;
@@ -2149,7 +2218,7 @@ int mktexpk(int argc, char **argv)
       record_output_handle(fd);
   }
 
-  start_redirection();
+  start_redirection(false);
 
   setval("NAME", argv[1]);
 
@@ -2162,18 +2231,26 @@ int mktexpk(int argc, char **argv)
   }
 
   if (FILESTRCASEEQ(getval("ps_to_pk"), "gsftopk")) {
-    if (system(expand_var("$ps_to_pk -t $NAME"))==0) {
+    if (KPSE_DEBUG_P(MKTEX_DEBUG)) {
+      printf("Test: system -> %d win32_system -> %d\n",
+	     system(expand_var("$ps_to_pk -t $NAME")),
+	     win32_system(expand_var("$ps_to_pk -t $NAME")));
+    }
+    if (win32_system(expand_var("$ps_to_pk -t $NAME"))==0) {
       cmd = expand_var("$ps_to_pk $NAME $DPI");
     }
   }
   else if (FILESTRCASEEQ(getval("ps_to_pk"), "ps2pk")) {
+    string pstmp;
     /* look for name in psfont.map.
        Can't use `kpse_open_file' because it exits if the file doesn't
        exits, and we have a special exit procedure which cleans up.  */
-    psmapfilename = setval_default("PSMAPFILE", 
-				   kpse_find_file("psfonts.map", 
-						  kpse_dvips_config_format, 
-						  false));
+    pstmp = kpse_find_file("ps2pk.map", kpse_dvips_config_format, false);
+    if (test_file('z', pstmp))
+      pstmp = kpse_find_file("psfonts.map", kpse_dvips_config_format, false);
+    psmapfilename = setval_default("PSMAPFILE", pstmp);
+    if (test_file('n', pstmp)) free(pstmp);
+
     if (test_file('n', psmapfilename)) {
       string pattern = concat3("^", getval("NAME"), "([ \t]|$)");
 
@@ -2214,6 +2291,7 @@ int mktexpk(int argc, char **argv)
 	if (strlen(aname) >= 2
 	    && aname[strlen(aname)-2] == '8' 
 	    && aname[strlen(aname)-1] == 'r')
+	  /* Guessing the name of the type1 font file as fallback: */
 	  aname[strlen(aname)-1] = 'a';
 	psname = kpse_find_file(aname, kpse_type1_format, false);
 	free(aname);
@@ -2246,11 +2324,18 @@ int mktexpk(int argc, char **argv)
   }
   
   if (test_file('z', cmd)) {
-    if (has_ttf2pk && system(expand_var("ttf2pk -t -q $NAME")) == 0) {
-      cmd = expand_var("ttf2pk -p $NAME $DPI");
+    if (has_ttf2pk) {
+      if (KPSE_DEBUG_P(MKTEX_DEBUG)) {
+	printf("Test: system -> %d win32_system -> %d\n",
+	       system(expand_var("ttf2pk -t -q $NAME")),
+	       win32_system(expand_var("ttf2pk -t -q $NAME")));
+      }
     }
-    else if (has_hbf2gf && system(expand_var("hbf2gf -t -q $NAME")) == 0) {
-      cmd = expand_var("hbf2gf -q -p $NAME $DPI");
+    if (has_ttf2pk && win32_system(expand_var("ttf2pk -t -q $NAME")) == 0) {
+      cmd = expand_var("ttf2pk -q $NAME $DPI");
+    }
+    else if (has_hbf2gf && win32_system(expand_var("hbf2gf -q -t $NAME")) == 0) {
+      cmd = expand_var("hbf2gf -g $NAME $DPI");
     }
   }
 
@@ -2287,6 +2372,9 @@ int mktexpk(int argc, char **argv)
       case 600:
 	setval("MODE", "ljfour");
 	break;
+      case 720:
+	setval("MODE", "epshi");
+	break;
       case 1270:
 	setval("MODE", "linoone");
 	break;
@@ -2329,8 +2417,9 @@ int mktexpk(int argc, char **argv)
 
   /* Now run metafont.
      Note that stdin has been redirected to "/dev/null" by `main'. */
-  retcode = system(cmd);
+  retcode = win32_system(cmd);
   if (retcode != 0) {
+      fprintf(stderr, "command %s return %d\n", cmd, retcode);
     if (retcode == -1) {
       perror("mf");
       mt_exit(1);
@@ -2352,7 +2441,7 @@ int mktexpk(int argc, char **argv)
   
   if (test_file('r', getval("GFNAME"))) {
     gfcmd = expand_var("gftopk ./$GFNAME $PKNAME");
-    retcode = system(gfcmd);
+    retcode = win32_system(gfcmd);
     if (retcode == -1) {
       perror("gftopk");
       mt_exit(1);
