@@ -30,10 +30,10 @@ extern MKTEXDLL void popd(void);
 extern char *peek_dir(int);
 extern MKTEXDLL void push_fd(int [3]);
 extern MKTEXDLL void pop_fd(void);
-extern void start_redirection(void);
+extern MKTEXDLL void start_redirection(boolean);
 #ifdef _WIN32
 /* extern void mt_exit(int); */
-extern BOOL sigint_handler(DWORD);
+extern MKTEXDLL BOOL sigint_handler(DWORD);
 #else
 extern void sigint_handler(int);
 #endif
