@@ -309,9 +309,9 @@ msdosdone:
                dosepsend += getc(f) * 256L ;
                dosepsend += getc(f) * 65536L ;
                dosepsend += getc(f) * 256L * 65536 ;
-               dosepsend += dosepsbegin;
                fseek(f, dosepsbegin, 0);
                c = getc(f);
+               dosepsend-- ;
             }
             else {
                rewind(f);
@@ -331,6 +331,7 @@ msdosdone:
                      (void)putc(c, bitfile) ;
                      while (1) {
                         c = getc(f) ;
+                        dosepsend-- ;
                         if (c == ' ')
                            (void)putc(c, bitfile) ;
                         else
@@ -340,39 +341,47 @@ msdosdone:
                         size = size * 10 + c - '0' ;
                         (void)putc(c, bitfile) ;
                         c = getc(f) ;
+                        dosepsend-- ;
                      }
                      while (c != '\r' && c != '\n') {
                         if (c == EOF)
                            error("! bad BeginBinary line in epsf") ;
                         (void)putc(c, bitfile) ;
                         c = getc(f) ;
+                        dosepsend-- ;
                      }
                      (void)putc(c, bitfile) ;
-		     if (c == '\r') { /* maybe we have a DOS-style text file */
-		        c = getc(f);
-			if (c == '\n')
-			   (void)putc(c, bitfile);
-			else
-			   ungetc(c, f);
-		     }
+                     if (c == '\r') { /* maybe we have a DOS-style text file */
+                        c = getc(f);
+                        if (c == '\n') {
+                           (void)putc(c, bitfile);
+                           dosepsend-- ;
+                        } else
+                           ungetc(c, f);
+                     }
                      for (; size>0; size--) {
                         c = getc(f) ;
+                        dosepsend-- ;
                         if (c == EOF)
                            error("! premature end of file in binary section") ;
                         (void)putc(c, bitfile) ;
                      }
                      c = getc(f) ;
+                     dosepsend-- ;
                      if (c == '\n' || c == '\r') {
                         (void)putc(c, bitfile) ;
 			if (c == '\r') { /* DOS-style text file? */
 			   c = getc(f);
+                           dosepsend-- ;
 			   if (c == '\n') {
 			      putc(c, bitfile);
 			      c = getc(f);
+                              dosepsend-- ;
 			   }
-			}
-			else
+			} else {
 			   c = getc(f) ;
+                           dosepsend-- ;
+                        }
                      }
                      if (c != '%')
                         error("! expected to see %%EndBinary at end of data") ;
@@ -383,14 +392,18 @@ msdosdone:
 			      c = getc(f);
 			      if (c != '\n')
 				 ungetc(c, f);
+                              else
+                                 dosepsend-- ;
 			   }
                            break ;
 			}
                         c = getc(f) ;
+                        dosepsend-- ;
                         if (c == EOF)
                            error("! premature end of file in binary section") ;
                      }
                      c = getc(f) ;
+                     dosepsend-- ;
                   }
                } else
                   bbmatch = begbinary ;
@@ -398,13 +411,17 @@ msdosdone:
                          /* skip comments */
 		         /* revised:  only skip %% and %! comments */
                   c = getc(f) ;
+                  dosepsend-- ;
                   if (c == '%' || c == '!') {
                      while ((c=getc(f))!=EOF) {
+                        dosepsend-- ;
                         if (c == '\n' || c == '\r') {
 			   if (c == '\r') { /* DOS-style text file? */
 			      c = getc(f);
 			      if (c != '\n')
 				 ungetc(c, f);
+                              else
+                                 dosepsend-- ;
 			   }
                            c = '\n' ;
                            break ;
@@ -428,17 +445,19 @@ msdosdone:
                }
                prevc = c ;
 /* begin DOS EPS code */
-               if (doseps && (ftell(f)>=dosepsend))
+               if (doseps && (dosepsend <= 0))
                   break;      /* stop at end of DOS EPS PostScript section */
 /* end DOS EPS code */
                c = getc(f) ;
+               dosepsend-- ;
                if (c == EOF)
                   break ;
                else if (c == '\r') {
 		  c = getc(f);
-		  if (c == '\n') /* DOS-style text file? */
+		  if (c == '\n') { /* DOS-style text file? */
 		     (void)putc('\r', bitfile);
-		  else
+                     dosepsend-- ;
+		  } else
 		     ungetc(c, f);
                   c = '\n' ;
 	       }
@@ -537,21 +556,22 @@ int T1Char P1C(int, c)
 {
   int tmpchr = c;
   if (shiftlowchars && curfnt->resfont) {
-    if (!(curfnt->chardesc[tmpchr].flags & EXISTS)) {
-      if ((tmpchr <= 0x20)&&(tmpchr>=0)) {
-	if (tmpchr < 0x0A) {
-	  tmpchr += 0xA1;
-	}
-	else{ 
-	  tmpchr += 0xA3;
-	}
+    if ((tmpchr <= 0x20)&&(tmpchr>=0)) {
+      if (tmpchr < 0x0A) {
+        tmpchr += 0xA1;
       }
-      else if (tmpchr == 0x7F) {
-	tmpchr = 0xC4;
+      else {
+        tmpchr += 0xA3;
       }
     }
+    else if (tmpchr == 0x7F) {
+      tmpchr = 0xC4;
+    }
   }
-  return tmpchr;
+  if (curfnt->chardesc[tmpchr].flags & EXISTS)
+    return c;
+  else
+    return tmpchr;
 }
 #endif
 
