@@ -225,7 +225,6 @@ char	*getenv();
 
 #else /* KPATHSEA */
 
-#include <kpathsea/c-auto.h>
 #include <kpathsea/config.h>
 #include <kpathsea/c-errno.h>
 #include <kpathsea/c-ctype.h>
@@ -235,14 +234,20 @@ char	*getenv();
 #include <kpathsea/tex-file.h>
 #include <kpathsea/tex-make.h>
 #include <kpathsea/variable.h>
+#include <c-auto.h>
 #include <signal.h>
 #include <fcntl.h>
 #include <setjmp.h>
 
-#if HAVE_SYS_TIME_H
+#if TIME_WITH_SYS_TIME
 # include <sys/time.h>
-#else
 # include <time.h>
+#else
+# if HAVE_SYS_TIME_H 
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
 #endif
 
 #ifndef HAVE_STRCHR
@@ -629,26 +634,34 @@ xrealloc P2C(char *, oldp, unsigned, size)
 	return mem;
 }
 
+#endif /* not KPATHSEA */
+
 
 /*
- *	Either create a new copy of the string or fail.
+ *	Get a single white-space-delimited argument (or fail).
  */
 
 char *
-xstrdup P1C(const char *, src)
+get_one_arg P1C(const char *, src)
 {
 	char		*dest;
-	unsigned	len;
+	const char	*arg_end;
+	const char	*p;
+	unsigned int	len;
 
-	len = strlen(src) + 1;
-	dest = xmalloc(len);
+	len = strlen(src);
+	p = memchr(src, ' ', len);
+	if (p != NULL) len = p - src;
+	p = memchr(src, '\t', len);
+	if (p != NULL) len = p - src;
+
+	dest = xmalloc(len + 1);
 	memcpy(dest, src, len);
+	dest[len] = '\0';
+
 	return dest;
 }
 
-#endif /* not KPATHSEA */
-
-
 #ifndef _AMIGA
 
 /*
@@ -1181,7 +1194,7 @@ getdefaults P1C(const char *, name)
 	    while (*p == ' ' || *p == '\t');
 	    switch (c) {
 		case 'H':
-		    config_file_header_path = xstrdup(p);
+		    config_file_header_path = get_one_arg(p);
 		    break;
 
 		case 'p':
@@ -1192,7 +1205,7 @@ getdefaults P1C(const char *, name)
 			p_tail = &p_head;	/* discard old list */
 
 		    p_node = xmalloc(sizeof *p_node);
-		    p_node->value = xstrdup(p);
+		    p_node->value = get_one_arg(p);
 		    *p_tail = p_node;
 		    p_tail = &p_node->next;
 		    break;
