@@ -72,7 +72,7 @@
 
 typedef char *encoding[256];
 
-#ifdef __STDC__
+#ifdef HAVE_PROTOTYPES
 int Type1OpenScalable(char **ev, struct _Font **ppFont, int flags,
 		      struct _FontEntry *entry, char *fileName,
 		      struct _FontScalable *vals, fsBitmapFormat format,
