@@ -679,10 +679,11 @@ parse_options P2C(int, argc,  string *, argv)
       kpathsea_debug |= atoi (optarg);
 
     } else if (ARGUMENT_IS ("progname")) {
-      if (!dumpoption) user_progname = optarg;
+      user_progname = optarg;
 
     } else if (ARGUMENT_IS (DUMP_OPTION)) {
-      dump_name = user_progname = optarg;
+      dump_name = optarg;
+      if (!user_progname) user_progname = optarg;
       dumpoption = true;
 
 #ifdef TeX
@@ -881,7 +882,8 @@ opennameok P3C(const_string, fname, const_string, check_var,
   {
     const_string base = xbasename (fname);
     /* Disallow .rhosts, .login, etc.  Allow .tex (for LaTeX).  */
-    if (*base == 0 || (*base == '.' && !STREQ (base, ".tex")))
+    if (base[0] == 0 ||
+        (base[0] == '.' && !IS_DIR_SEP(base[1]) && !STREQ (base, ".tex")))
       return false;
   }
 #else
@@ -898,17 +900,17 @@ opennameok P3C(const_string, fname, const_string, check_var,
        fname begins the TEXMFOUTPUT, and is followed by / */
     if (!texmfoutput || *texmfoutput == '\0'
         || fname != strstr (fname, texmfoutput)
-        || fname[strlen(texmfoutput)] != DIR_SEP)
+        || !IS_DIR_SEP(fname[strlen(texmfoutput)]))
       return false;
   }
   /* For all pathnames, we disallow "../" at the beginning or "/../"
      anywhere.  */
-  if (fname[0] == '.' && fname[1] == '.' && fname[2] == DIR_SEP) {
+  if (fname[0] == '.' && fname[1] == '.' && IS_DIR_SEP(fname[2])) {
     return false;
   } else {
     const_string dotpair = strstr (fname, "..");
     /* If dotpair[2] == DIR_SEP, then dotpair[-1] is well-defined. */
-    if (dotpair && dotpair[2] == DIR_SEP && dotpair[-1] == DIR_SEP)
+    if (dotpair && IS_DIR_SEP(dotpair[2]) && IS_DIR_SEP(dotpair[-1]))
       return false;
   }
 
@@ -1414,7 +1416,7 @@ connection with the use or performance of this software.
 #define TWEXP_28 (1.0/268435456.0)
 
 integer
-ztakefraction P2C(integer, p, integer, q)	/* Approximate p*q/2^28 */
+ztakefraction P2C(integer, p, integer, q)     /* Approximate p*q/2^28 */
 {	register double d;
 	register integer i;
 	d = (double)p * (double)q * TWEXP_28;
@@ -1500,8 +1502,7 @@ zmakefraction P2C(integer, p, integer, q)	/* Approximate 2^28*p/q */
    and 2^15/1.  No pair of integers less than 2^31 has such a ratio.
 */
 integer
-zmakescaled(p,q)		/* Approximate 2^16*p/q */
-integer p,q;
+zmakescaled P2C(integer, p, integer, q)		/* Approximate 2^16*p/q */
 {	register double d;
 	register integer i;
 #ifdef DEBUG
@@ -1777,10 +1778,8 @@ updatescreen P1H(void)
    the background color.  */
 
 void
-blankrectangle P4C(screencol, left,
-                   screencol, right,
-                   screenrow, top,
-                   screenrow, bottom)
+blankrectangle P4C(screencol, left, screencol, right,
+                   screenrow, top, screenrow, bottom)
 {
   if (mfwp->mfwsw_blankrect)
     (*mfwp->mfwsw_blankrect) (left, right, top, bottom);
@@ -1792,10 +1791,8 @@ blankrectangle P4C(screencol, left,
    switch colors.  This goes on for VECTOR_SIZE transitions.  */
 
 void
-paintrow P4C(screenrow, row,
-             pixelcolor, init_color,
-             transspec, transition_vector,
-             screencol, vector_size)
+paintrow P4C(screenrow, row, pixelcolor, init_color,
+             transspec, transition_vector, screencol, vector_size)
 {
   if (mfwp->mfwsw_paintrow)
     (*mfwp->mfwsw_paintrow) (row, init_color, transition_vector, vector_size);
