@@ -1,9 +1,11 @@
 #include <kpathsea/c-auto.h>
-#include <kpathsea/c-std.h>
-#include <kpathsea/c-unistd.h>
+#include <kpathsea/c-fopen.h>
 #include <kpathsea/c-limits.h>
 #include <kpathsea/c-memstr.h>
-#include <kpathsea/c-fopen.h>
+#include <kpathsea/c-proto.h>
+#include <kpathsea/c-std.h>
+#include <kpathsea/c-unistd.h>
+#include <kpathsea/c-vararg.h>
 #include <kpathsea/getopt.h>
 #include <time.h>
 
@@ -81,28 +83,10 @@ typedef struct {
 #define GET_F2DOT14()   GET_TYPE(TTF_F2DOT14)
 
 #define NTABS           24
-#define AND             ,
-
-#define FAIL(S) do {                                       \
-    fprintf(stderr, "Error: ");                            \
-    if (filename != 0)                                     \
-        fprintf(stderr, "file `%s': ", filename);          \
-    fprintf(stderr, S);                                    \
-    fprintf(stderr, "\n");                                 \
-    _exit(-1);                                             \
-} while (0)
-
-#define WARN(S) do {                                       \
-    fprintf(stderr, "Warning: ");                          \
-    if (filename != 0)                                     \
-        fprintf(stderr, "file `%s': ", filename);          \
-    fprintf(stderr, S);                                    \
-    fprintf(stderr, "\n");                                 \
-} while (0)
 
 #define CHECK_BUF(size, buf_size)                          \
     if ((size) >= buf_size - 2)                            \
-        FAIL("buffer overflow [%i bytes]" AND buf_size)
+        fail("buffer overflow [%i bytes]", buf_size)
 
 #define APPEND_CHAR_TO_BUF(c, p, buf, buf_size) do {       \
     if (c == 9)                                            \
@@ -137,6 +121,8 @@ typedef struct {
 #define XTALLOC(n, t) ((t *) xmalloc ((n) * sizeof (t)))
 #define XFREE(P)            do { if (P != 0) free(P); } while (0)
 
+#define strend(S)           strchr(S, 0)
+
 #define NMACGLYPHS      258
 #define MAX_CHAR_NUM    256
 #define ENC_BUF_SIZE    1024
@@ -192,11 +178,39 @@ char *enc_names[MAX_CHAR_NUM];
 
 #include "macnames.c"
 
+void fail(char *fmt,...)
+{
+    va_list args;
+    va_start(args, fmt);
+    fprintf(stderr, "\nError: ttf2afm");
+    if (filename)
+        fprintf(stderr, "(file %s)", filename);
+    fprintf(stderr, ": ");
+    vfprintf(stderr, fmt, args);
+    fprintf(stderr, "\n");
+    va_end(args);
+    exit(-1);
+}
+
+void warn(char *fmt,...)
+{
+    va_list args;
+    va_start(args, fmt);
+    fprintf(stderr, "\nWarning: ttf2afm");
+    if (filename)
+        fprintf(stderr, "(file %s)", filename);
+    fprintf(stderr, ": ");
+    vfprintf(stderr, fmt, args);
+    fprintf(stderr, "\n");
+    va_end(args);
+}
+
+
 void *xmalloc(unsigned long size)
 {
     void *p = (void *)malloc(size);
     if (p == 0)
-        FAIL("malloc() failed (%lu bytes)" AND (unsigned long)size);
+        fail("malloc() failed (%lu bytes)", (unsigned long)size);
     return p;
 }
 
@@ -204,21 +218,21 @@ FILE *xfopen(char *name, char *mode)
 {
     FILE *f = fopen(name, mode);
     if (f == 0)
-        FAIL("fopen() failed");
+        fail("fopen() failed");
     return f;
 }
 
 void xfclose(FILE *f)
 {
   if (fclose (f) != 0)
-        FAIL("fclose() failed");
+        fail("fclose() failed");
 }
 
 long xftell(FILE *f)
 {
     long offset = ftell(f);
     if (offset < 0)
-        FAIL("ftell() failed");
+        fail("ftell() failed");
     return offset;
 }
 
@@ -226,7 +240,7 @@ int xgetc(FILE *stream)
 {
     int c = getc(stream);
     if (c < 0 && c != EOF)
-        FAIL("getc() failed");
+        fail("getc() failed");
     return c;
 }
 
@@ -242,7 +256,7 @@ long getnum(int s)
     int c;
     while (s > 0) {
         if ((c = xgetc(fontfile)) < 0)
-            FAIL("unexpected EOF");
+            fail("unexpected EOF");
         i = (i << 8) + c;
         s--;
     }
@@ -264,15 +278,15 @@ void seek_tab(char *name, TTF_LONG offset)
 {
     dirtab_entry *p = name_lookup(name);
     if (p == 0)
-        FAIL("can't find table `%s'" AND name);
+        fail("can't find table `%s'", name);
     if (fseek(fontfile, p->offset + offset, SEEK_SET) < 0)
-        FAIL("fseek() failed while reading `%s' table" AND name);
+        fail("fseek() failed while reading `%s' table", name);
 }
 
 void seek_off(char *name, TTF_LONG offset)
 {
     if (fseek(fontfile, offset, SEEK_SET) < 0)
-        FAIL("fseek() failed while reading `%s' table" AND name);
+        fail("fseek() failed while reading `%s' table", name);
 }
 
 void store_kern_value(TTF_USHORT i, TTF_USHORT j, TTF_FWORD v)
@@ -321,7 +335,7 @@ void enc_getline()
     int c;
 restart:
     if (ENC_EOF())
-        FAIL("unexpected end of file");
+        fail("unexpected end of file");
     p = enc_line;
     do {
         c = ENC_GETCHAR();
@@ -338,10 +352,10 @@ void read_encoding(char *encname)
     int i;
     filename = encname;
     if ((encfile = xfopen(encname, FOPEN_RBIN_MODE)) == 0)
-        FAIL("can't open encoding file for reading");
+        fail("can't open encoding file for reading");
     enc_getline();
     if (*enc_line != '/' || (r = strchr(enc_line, '[')) == NULL)
-        FAIL("invalid encoding vector: name or `[' missing:\n%s" AND enc_line);
+        fail("invalid encoding vector: name or `[' missing:\n%s", enc_line);
     for (i = 0; i < MAX_CHAR_NUM; i++)
         enc_names[i] = notdef;
     if (r[1] == 32)
@@ -357,14 +371,14 @@ void read_encoding(char *encname)
             if (strcmp(buf, notdef) != 0)
                 enc_names[names_count] = xstrdup(buf);
             if (names_count++ >= MAX_CHAR_NUM)
-                FAIL("encoding vector contains more than %i names" AND
+                fail("encoding vector contains more than %i names",
                      (int)MAX_CHAR_NUM);
         }
         if (*r != 10 && *r != '%')
             if (strncmp(r, "] def", strlen("] def")) == 0)
                 goto done;
             else
-                FAIL("invalid encoding vector: a name or `] def' expected:\n%s" AND enc_line);
+                fail("invalid encoding vector: a name or `] def' expected:\n%s", enc_line);
         enc_getline();
         r = enc_line;
     }
@@ -440,7 +454,7 @@ void read_font()
         for (pm = mtx_tab; pm - mtx_tab < l; pm++)
             pm->index = GET_USHORT();
         if ((pd = name_lookup("post")) == 0)
-            FAIL("can't find table `post'");
+            fail("can't find table `post'");
         n = pd->length - (xftell(fontfile) - pd->offset);
         ps_glyphs_buf = XTALLOC(n + 1, char);
         for (p = ps_glyphs_buf; p - ps_glyphs_buf < n;) {
@@ -454,19 +468,19 @@ void read_font()
             else {
                 k = pm->index - NMACGLYPHS;
                 for (p = ps_glyphs_buf; k > 0; k--)
-                    p = (char *)strchr(p, 0) + 1;
+                    p = (char *)strend(p) + 1;
                 pm->name = p;
             }
         }
         break;
     default:
-        FAIL("unsupported format (%8X) of `post' table" AND post_format);
+        fail("unsupported format (%8X) of `post' table", post_format);
     }
     seek_tab("loca", 0);
     for (pm = mtx_tab; pm - mtx_tab < nglyphs; pm++)
         pm->offset = (loca_format == 1 ? GET_ULONG() : GET_USHORT() << 1);
     if ((pd = name_lookup("glyf")) == 0)
-        FAIL("can't find table `glyf'");
+        fail("can't find table `glyf'");
     for (n = pd->offset, pm = mtx_tab; pm - mtx_tab < nglyphs; pm++)
         if (pm->offset != (pm + 1)->offset) {
             seek_off("glyf", n + pm->offset);
@@ -641,7 +655,7 @@ void print_afm(char *date, char *fontname)
                     ncharmetrics++;
             }
             else
-                WARN("glyph `%s' not found" AND *pe);
+                warn("glyph `%s' not found", *pe);
         }
         fprintf(outfile, "\nStartCharMetrics %u\n", ncharmetrics);
         for (idx = mtx_index; idx - mtx_index < MAX_CHAR_NUM; idx++) {
@@ -754,18 +768,18 @@ void print_encoding(char *fontname)
     }
     enc_name = XTALLOC(strlen(b_name) + 20, char);
     strcpy(enc_name, b_name);
-    end_enc_name = strchr(enc_name, 0);
+    end_enc_name = strend(enc_name);
     for (e = cmap_tab; e - cmap_tab < netabs; e++) {
         seek_off("cmap", cmap_offset + e->offset);
         format = GET_USHORT();
         if (format != 0 && format != 4) {
-            WARN("format %i of encoding subtable unsupported" AND (int)format);
+            warn("format %i of encoding subtable unsupported", (int)format);
             continue;
         }
         sprintf(end_enc_name, ".e%i%i", 
                 (int)e->platform_id, (int)e->encoding_id);
         if ((file = xfopen(enc_name, FOPEN_W_MODE)) == 0)
-            FAIL("cannot open file for writting (%s)\n" AND enc_name);
+            fail("cannot open file for writting (%s)\n", enc_name);
         fprintf(file, "%% Encoding table from font file %s\n", fontname);
         fprintf(file, "%% Platform ID %i\n", (int)e->platform_id);
         fprintf(file, "%% Encoding ID %i\n", (int)e->encoding_id);
@@ -831,7 +845,7 @@ void print_encoding(char *fontname)
             }
             if (last_sep == 0)
                 fprintf(file, "\n");
-            fprintf(file, "%% Characters with code larger than %i have been ignored:\n" AND 
+            fprintf(file, "%% Characters with code larger than %i have been ignored:\n", 
                            (int)(MAX_CHAR_NUM - 1));
             for (s = seg_tab; s - seg_tab < segCount; s++) {
                 if (s->endCode < MAX_CHAR_NUM)
@@ -858,13 +872,14 @@ void print_encoding(char *fontname)
 void usage()
 {
     filename = 0;
-    fprintf(stderr, "Usage: ttf2afm [-a] [-c] [-i] [-e encoding]  [-o output] fontfile\n"
-                    "-a: print all glyphs\n"
-                    "-c: print encoding tables\n"
-                    "-i: print glyph names in form `index123'\n"
-                    "-e encoding: use encoding from file `encoding'\n"
-                    "-o output: output to file `output' instead of stdout\n"
-                    "fontfile: the TrueType font\n");
+    fprintf(stderr,
+        "Usage: ttf2afm [-a] [-c encname] [-i] [-e encoding]  [-o output] fontfile\n"
+            "-a: print all glyphs\n"
+            "-c encname: print encoding tables to `encname.eMN'\n"
+            "-i: print glyph names in form `index123'\n"
+            "-e encoding: use encoding from file `encoding'\n"
+            "-o output: output to file `output' instead of stdout\n"
+            "fontfile: the TrueType font\n");
     _exit(-1);
 }
 
@@ -895,7 +910,7 @@ int main(int argc, char **argv)
             filename = optarg;
             outfile = xfopen(filename, FOPEN_W_MODE);
             if (outfile == 0)
-                FAIL("cannot open file for writting");
+                fail("cannot open file for writting");
             break;
         default:
             usage();
@@ -906,7 +921,7 @@ int main(int argc, char **argv)
     *(char *)strchr(date, '\n') = 0;
     filename = argv[optind];
     if ((fontfile = fopen(filename, FOPEN_RBIN_MODE)) == 0)
-        FAIL("can't open font file for reading");
+        fail("can't open font file for reading");
     read_font();
     if (outfile == 0)
         outfile = stdout;
