@@ -143,12 +143,12 @@ static ttfenc_entry *ttfenc_tab;
 static short *glyph_index;
 static int ncmaptabs;
 static cmap_entry *cmap_tab;
-static name_record *names_tab;
-static int names_record_num;
-static char *names_storage;
-static int names_storage_size;
+static name_record *name_tab;
+static int name_record_num;
+static char *name_storage;
+static int name_storage_size;
 static tabdir_entry *dir_tab;
-static char *glyph_names_buf;
+static char *glyph_name_buf;
 static int chkcount;
 static FILE *ttf_file;
 static char id_str[] = "index";
@@ -212,7 +212,7 @@ static long getnum(int s)
     int c;
     while (s > 0) {
         if ((c = TTF_GETCHAR()) < 0)
-            FAIL("unexpected EOF");
+            pdftex_fail("unexpected EOF");
         i = (i << 8) + c;
         s--;
     }
@@ -248,7 +248,7 @@ static tabdir_entry *name_lookup(char *s, boolean required)
             break;
     if (tab - dir_tab == ntabs) {
         if (required)
-            FAIL("can't find table `%s'" AND s);
+            pdftex_fail("can't find table `%s'", s);
         else
             tab = 0;
     }
@@ -258,15 +258,15 @@ static tabdir_entry *name_lookup(char *s, boolean required)
 static tabdir_entry *seek_tab(char *name, TTF_LONG offset)
 {
     tabdir_entry *tab = name_lookup(name, true);
-    sprintf(printf_buf, "%s while reading table `%s'", filename, name);
-    xfseek(INFILE, tab->offset + offset, SEEK_SET, printf_buf);
+    sprintf(print_buf, "%s while reading table `%s'", filename, name);
+    xfseek(INFILE, tab->offset + offset, SEEK_SET, print_buf);
     return tab;
 }
 
 static void seek_off(char *name, TTF_LONG offset)
 {
-    sprintf(printf_buf, "%s while reading table %s", filename, name);
-    xfseek(INFILE, offset, SEEK_SET, printf_buf);
+    sprintf(print_buf, "%s while reading table %s", filename, name);
+    xfseek(INFILE, offset, SEEK_SET, print_buf);
 }
 
 static void copy_encoding()
@@ -284,22 +284,66 @@ static void copy_encoding()
     }
 }
 
-static void get_fontname()
+#define TTF_APPEND_BYTE(B)\
+do {\
+    if (name_tab[i].platform_id == 3)\
+        *q++ = 0;\
+    *q++ = B;\
+} while (0)
+
+static void check_name()
 {
     int i, l;
-    char *r, buf[1024], *p;
-    for (i = 0; i < names_record_num; i++)
-        if (names_tab[i].platform_id == 1 && 
-            names_tab[i].encoding_id == 0 &&
-            names_tab[i].name_id == 6) {
-            r = names_storage + names_tab[i].offset;
-            l = names_tab[i].length;
-            for (p = buf; l > 0; l--)
+    static char *name_prefix = "Embedded";
+    char  buf[1024], *p, *r, *s, *t;
+    int new_name_storage_size = name_storage_size +
+        name_record_num*7*2*(strlen(fm_cur->prefix) + 1);
+    char *new_name_storage = XTALLOC(new_name_storage_size, char),
+        *q = new_name_storage;
+    for (i = 0; i < name_record_num; i++) {
+        if (name_tab[i].platform_id == 1 && 
+/*              name_tab[i].encoding_id == 0 && */
+            name_tab[i].name_id == 6) {
+            r = name_storage + name_tab[i].offset;
+            l = name_tab[i].length;
+            strcpy(buf, name_prefix);
+            p = strend(buf);
+            *p++ = '-';
+            for (; l > 0; l--)
                 *p++ = *r++;
             *p = 0;
             font_keys[FONTNAME_CODE].value.s = xstrdup(buf);
             font_keys[FONTNAME_CODE].valid = true;
         }
+        r = name_storage + name_tab[i].offset;
+        l = name_tab[i].length;
+        name_tab[i].offset = q - new_name_storage;
+        t = q;
+        switch (name_tab[i].name_id) {
+        case 1: case 3: case 4: case 6:
+            s = name_prefix;
+            for (; *s != 0; s++)
+                TTF_APPEND_BYTE(*s);
+        }
+        switch (name_tab[i].name_id) {
+        case 1: case 4:
+            TTF_APPEND_BYTE(' ');
+            break;
+        case 3:
+            TTF_APPEND_BYTE(':');
+            TTF_APPEND_BYTE(' ');
+            break;
+        case 6:
+            TTF_APPEND_BYTE('-');
+            break;
+        }
+        for (; l > 0; l--)
+            *q++ = *r++;
+        name_tab[i].length = q - t;
+    }
+    XFREE(name_storage);
+    name_storage = new_name_storage;
+    name_storage_size = q - new_name_storage;
 }
 
 static void read_name()
@@ -307,23 +351,23 @@ static void read_name()
     int i;
     tabdir_entry *tab = seek_tab("name", TTF_USHORT_SIZE);
     char *p;
-    names_record_num = GET_USHORT();
-    names_tab = XTALLOC(names_record_num, name_record);
-    names_storage_size = tab->length - 
-         (3*TTF_USHORT_SIZE + names_record_num*6*TTF_USHORT_SIZE);
-    names_storage = XTALLOC(names_storage_size, char);
+    name_record_num = GET_USHORT();
+    name_tab = XTALLOC(name_record_num, name_record);
+    name_storage_size = tab->length - 
+         (3*TTF_USHORT_SIZE + name_record_num*6*TTF_USHORT_SIZE);
+    name_storage = XTALLOC(name_storage_size, char);
     SKIP(TTF_USHORT_SIZE);
-    for (i = 0; i < names_record_num; i++) {
-        names_tab[i].platform_id = GET_USHORT();
-        names_tab[i].encoding_id = GET_USHORT();
-        names_tab[i].language_id = GET_USHORT();
-        names_tab[i].name_id = GET_USHORT();
-        names_tab[i].length = GET_USHORT();
-        names_tab[i].offset = GET_USHORT();
+    for (i = 0; i < name_record_num; i++) {
+        name_tab[i].platform_id = GET_USHORT();
+        name_tab[i].encoding_id = GET_USHORT();
+        name_tab[i].language_id = GET_USHORT();
+        name_tab[i].name_id = GET_USHORT();
+        name_tab[i].length = GET_USHORT();
+        name_tab[i].offset = GET_USHORT();
     }
-    for (p = names_storage; p - names_storage < names_storage_size;)
-         *p++ = GET_CHAR();
-    get_fontname();
+    for (p = name_storage; p - name_storage < name_storage_size; p++)
+         *p = GET_CHAR();
+    check_name();
 }
 
 static void read_mapx()
@@ -334,6 +378,7 @@ static void read_mapx()
     glyph_tab = XTALLOC(1 + (glyphs_count = GET_USHORT()), glyph_entry);
     for (glyph = glyph_tab; glyph - glyph_tab < glyphs_count; glyph++) {
         glyph->newindex = -1;
+        glyph->newoffset = 0;
         glyph->name_index = 0;
         glyph->name = notdef;
     }
@@ -421,8 +466,8 @@ static void read_post()
         for (glyph = glyph_tab; glyph - glyph_tab < nnames; glyph++)
             glyph->name_index = GET_USHORT();
         length = tab->length - (xftell(INFILE, filename) - tab->offset);
-        glyph_names_buf = XTALLOC(length, char);
-        for (p = glyph_names_buf; p - glyph_names_buf < length;) {
+        glyph_name_buf = XTALLOC(length, char);
+        for (p = glyph_name_buf; p - glyph_name_buf < length;) {
             for (k = GET_BYTE(); k > 0; k--)
                 *p++ = GET_CHAR();
             *p++ = 0;
@@ -431,17 +476,17 @@ static void read_post()
             if (glyph->name_index < NMACGLYPHS)
                 glyph->name = mac_glyph_names[glyph->name_index];
             else {
-                p = glyph_names_buf;
+                p = glyph_name_buf;
                 k = glyph->name_index - NMACGLYPHS;
                 for (; k > 0; k--)
-                    p = (char *)strchr(p, 0) + 1;
+                    p = strend(p) + 1;
                 glyph->name = p;
             }
         }
         break;
     default:
-        FAIL("unsupported format (%8X) of `post' table"
-             AND (unsigned int) post_format);
+        pdftex_fail("unsupported format (%8X) of `post' table"
+, (unsigned int) post_format);
     }
 }
 
@@ -463,7 +508,7 @@ static void read_font()
     long version;
     tabdir_entry *tab;
     if ((version= GET_FIXED()) != 0x00010000)
-        FAIL("unsupport version %8lx; can handle only version 1.0" AND version);
+        pdftex_fail("unsupport version %8lx; can handle only version 1.0", version);
     dir_tab = XTALLOC(ntabs = GET_USHORT(), tabdir_entry);
     SKIP(3*TTF_USHORT_SIZE);
     for (tab = dir_tab; tab - dir_tab < ntabs; tab++) {
@@ -526,12 +571,26 @@ static void byte_encoding()
         }
         else {
             if (e->name != notdef)
-                WARN("glyph `%s' has been mapped to `%s' in `byte encoding' cmap table" AND
-                     e->name AND notdef);
+                pdftex_warn("glyph `%s' has been mapped to `%s' in `byte_encoding' cmap table",
+                     e->name, notdef);
             PUT_BYTE(0); /* notdef */
         }
 }
 
+#define TRIMMED_TABLE_MAP_LENGTH (TTF_USHORT_SIZE*(5 + MAX_CHAR_NUM))
+
+static void trimmed_table_map()
+{
+    ttfenc_entry *e;
+    PUT_USHORT(6);  /* format number (6): trimmed table mapping */
+    PUT_USHORT(TRIMMED_TABLE_MAP_LENGTH);
+    PUT_USHORT(0);  /* version number (0) */
+    PUT_USHORT(0);  /* first character code */
+    PUT_USHORT(MAX_CHAR_NUM);  /* number of character code in table */
+    for (e = ttfenc_tab; e - ttfenc_tab < MAX_CHAR_NUM; e++)
+        PUT_USHORT(e->newindex);
+}
+
 #define SEG_MAP_DELTA_LENGTH ((16 + MAX_CHAR_NUM)*TTF_USHORT_SIZE)
 
 static void seg_map_delta()
@@ -578,15 +637,20 @@ void static select_cmap()
         cur_offset += CMAP_ENTRY_LENGTH;
         xfseek(INFILE, cmap_offset + offset, SEEK_SET, filename);
         format = GET_USHORT();
-        if (format == 0 || format == 4) {
+        if (format == 0 || format == 4 || format == 6) {
             cmap_tab[ncmaptabs].platform_id  = pid;
-            cmap_tab[ncmaptabs].encoding_id  = eid;
+/*          cmap_tab[ncmaptabs].encoding_id  = eid; */
+            cmap_tab[ncmaptabs].encoding_id  = 0; /* symbol-like encoding; don't use any code page */
             cmap_tab[ncmaptabs].offset       = offset;
             cmap_tab[ncmaptabs].format       = format;
+            if (format == 0 && new_glyphs_count > 256) {
+                pdftex_warn("`byte encoding' subtable of cmap has been ignored (cannot map more than 254 characters");
+                continue;
+            }
             ncmaptabs++;
         }
         else
-            WARN("format %i of encoding subtable unsupported" AND (int)format);
+            pdftex_warn("format %i of encoding subtable unsupported", (int)format);
     }
 }
 
@@ -609,8 +673,11 @@ static void write_cmap()
         case 4: 
             offset +=  SEG_MAP_DELTA_LENGTH;
             break;
+        case 6: 
+            offset +=  TRIMMED_TABLE_MAP_LENGTH;
+            break;
         default:
-            FAIL("writting cmap: format %i unsupported" AND (int)ce->format);
+            pdftex_fail("invalid format (it should not have happened)");
         }
         PUT_USHORT(ce->platform_id);
         PUT_USHORT(ce->encoding_id);
@@ -624,12 +691,37 @@ static void write_cmap()
         case 4: 
             seg_map_delta();
             break;
+        case 6: 
+            trimmed_table_map();
+            break;
         }
     }
     SET_CHKSM();
     XFREE(cmap_tab);
 }
 
+static void write_name()
+{
+    int i;
+    char *p;
+    tabdir_entry *tab = name_lookup("name", true);
+    RESET_CHKSM();
+    PUT_USHORT(0); /* Format selector */
+    PUT_USHORT(name_record_num);
+    PUT_USHORT(3*TTF_USHORT_SIZE + name_record_num*6*TTF_USHORT_SIZE);
+    for (i = 0; i < name_record_num; i++) {
+        PUT_USHORT(name_tab[i].platform_id);
+        PUT_USHORT(name_tab[i].encoding_id);
+        PUT_USHORT(name_tab[i].language_id);
+        PUT_USHORT(name_tab[i].name_id);
+        PUT_USHORT(name_tab[i].length);
+        PUT_USHORT(name_tab[i].offset);
+    }
+    for (p = name_storage; p - name_storage < name_storage_size; p++)
+         PUT_CHAR(*p);
+    SET_CHKSM();
+}
+
 static void copy_dirtab()
 {
     tabdir_entry *tab;
@@ -709,7 +801,7 @@ static void write_glyf()
     SET_CHKSM();
 }
 
-static void reindex_glyphs(boolean subset)
+static void reindex_glyphs()
 {
     ttfenc_entry *e;
     glyph_entry *glyph;
@@ -737,16 +829,15 @@ static void reindex_glyphs(boolean subset)
                             break;
             }
             if (glyph - glyph_tab < glyphs_count) {
-                if (subset) {
+                if (glyph->newindex < 0) {
                     glyph_index[new_glyphs_count] = glyph - glyph_tab;
-                    glyph->newindex = e->newindex = new_glyphs_count;
+                    glyph->newindex = new_glyphs_count;
                     new_glyphs_count++;
                 }
-                else
-                    e->newindex = glyph - glyph_tab;
+                e->newindex = glyph->newindex;
             }
             else
-                WARN("glyph `%s' not found" AND e->name);
+                pdftex_warn("glyph `%s' not found", e->name);
         }
     }
 }
@@ -822,6 +913,32 @@ static void write_mapx()
     SET_CHKSM();
 }
 
+static void write_OS2()
+{
+    tabdir_entry *tab = seek_tab("OS/2", 0);
+    TTF_USHORT version;
+    RESET_CHKSM();
+    version = GET_USHORT();
+    if (version != 0x0000 && version != 0x0001)
+        pdftex_fail("unknown verssion of OS/2 table (%4X)", version);
+    PUT_USHORT(0x0001); /* fix version to 1*/
+    ncopy(2*TTF_USHORT_SIZE + 13*TTF_SHORT_SIZE + 10*TTF_BYTE_SIZE);
+    SKIP(4*TTF_ULONG_SIZE); /* ulUnicodeRange 1--4 */
+    PUT_ULONG(0x00000003); /* Basic Latin + Latin-1 Supplement (0x0000--0x00FF) */
+    PUT_ULONG(0x10000000); /* Private Use (0xE000--0xF8FF) */
+    PUT_ULONG(0x00000000);
+    PUT_ULONG(0x00000000);
+    ncopy(4*TTF_CHAR_SIZE + TTF_USHORT_SIZE); /* achVendID + fsSelection */
+    SKIP(2*TTF_USHORT_SIZE);
+    PUT_USHORT(0x0000); /* usFirstCharIndex */
+    PUT_USHORT(0xF0FF); /* usLastCharIndex */
+    ncopy(5*TTF_USHORT_SIZE);
+    /* for version 0 the OS/2 table ends here, the rest is for version 1 */ 
+    PUT_ULONG(0x80000000); /* Symbol Character Set---don't use any code page */
+    PUT_ULONG(0x00000000);
+    SET_CHKSM();
+}
+
 static void write_post()
 {
     int i;
@@ -866,7 +983,6 @@ static void init_font(int n)
 static void subset_font()
 {
     init_font(new_ntabs);
-    copytab("OS/2");
     if (name_lookup("PCLT", false) != 0)
         copytab("PCLT");
     if (name_lookup("fpgm", false) != 0)
@@ -875,14 +991,15 @@ static void subset_font()
         copytab("cvt ");
     if (name_lookup("prep", false) != 0)
         copytab("prep");
-    reindex_glyphs(true);
+    reindex_glyphs();
     write_glyf();
     write_loca();
+    write_OS2();
     write_head();
     write_hhea();
     write_htmx();
     write_mapx();
-    copytab("name");
+    write_name();
     write_post();
     write_cmap();
     copy_dirtab();
@@ -893,12 +1010,7 @@ static void copy_font()
     tabdir_entry *tab;
     init_font(ntabs);
     for (tab = dir_tab; tab - dir_tab < ntabs; tab++)
-        if (is_reencoded() && (strncmp(tab->tag, "cmap", 4) == 0)) {
-            reindex_glyphs(false);
-            write_cmap();
-        }
-        else
-            copytab(tab->tag);
+        copytab(tab->tag);
     copy_dirtab();
 }
 
@@ -907,24 +1019,24 @@ void writettf()
     filename = fm_cur->ff_name;
     packfilename(maketexstring(filename), getnullstr(), getnullstr());
     if (!TTF_OPEN()) {
-        WARN("cannot open TrueType font file for reading");
+        pdftex_warn("cannot open TrueType font file for reading");
         font_file_not_found = true;
         return;
     }
     if (is_subsetted() && !is_reencoded()) {
-        WARN("encoding vector required for TrueType font subsetting");
+        pdftex_warn("encoding vector required for TrueType font subsetting");
         return;
     }
-    TEX_PRINTF("<%s" AND fm_cur->ff_name);
-    glyph_names_buf = 0;
+    tex_printf("<%s", fm_cur->ff_name);
+    glyph_name_buf = 0;
     new_glyphs_count = 2;
     new_ntabs = DEFAULT_NTABS;
     dir_tab = 0;
     glyph_tab = 0;
     glyph_index = 0;
-    glyph_names_buf = 0;
-    names_tab = 0;
-    names_storage = 0;
+    glyph_name_buf = 0;
+    name_tab = 0;
+    name_storage = 0;
     ttfenc_tab = 0;
     read_font();
     if (is_included()) {
@@ -945,9 +1057,9 @@ void writettf()
     XFREE(dir_tab);
     XFREE(glyph_tab);
     XFREE(glyph_index);
-    XFREE(glyph_names_buf);
-    XFREE(names_tab);
-    XFREE(names_storage);
+    XFREE(glyph_name_buf);
+    XFREE(name_tab);
+    XFREE(name_storage);
     TTF_CLOSE();
-    TEX_PRINTF(">");
+    tex_printf(">");
 }
